/*-Includes-------------------------------------------------------------------+
 |                                                                            |
 +----------------------------------------------------------------------------*/

#include <windows.h>
#include <time.h>
#include <math.h>

#include <sstream>

#include <SDL.h>

#pragma warning ( disable:4786 )
#include <list>
#include <vector>

#pragma comment( lib, "sdl.lib" )
#pragma comment( lib, "sdlmain.lib" )



/*-Audio-Stuff----------------------------------------------------------------+
 |                                                                            |
 +----------------------------------------------------------------------------*/

static Uint8 *audio_chunk;  
static Uint32 audio_len;
static Uint8 *audio_pos;

void my_audio_callback( void *userdata, Uint8 *stream, int len )
{
  /* Only play if we have data left */    
  if ( audio_len == 0 ) return;      
  /* Mix as much data as possible */
  len = ( len > (int)audio_len ? audio_len : len );

  SDL_MixAudio(stream, audio_pos, len, SDL_MIX_MAXVOLUME);
  audio_pos += len;      
  audio_len -= len;    
}


struct wav_t
{
  SDL_AudioSpec wav_spec;
  Uint32 wav_length;
  Uint8 *wav_buffer;

  wav_t()
  : wav_length( 0 ),
    wav_buffer( 0 )
  {}
};

typedef std::vector<wav_t> wavvector;
wavvector allewavs;



/*-LoadWave-------------------------------------------------------------------+
 |                                                                            |
 +----------------------------------------------------------------------------*/

unsigned int LoadWave( const std::string& file )
{
  wav_t w;

  /* Load the WAV */
  if( SDL_LoadWAV( file.c_str(), &w.wav_spec, &w.wav_buffer, &w.wav_length) == NULL )
  {
  #ifdef _DEBUG
      std::ostringstream os;
      os << SDL_GetError() <<"\n";
      OutputDebugString( os.str().c_str() );
  #endif
    return 0;
  }
  w.wav_spec.callback=my_audio_callback;
  w.wav_spec.userdata=NULL;

  allewavs.push_back( w );
  return allewavs.size() - 1;
}



/*-PlayWave-------------------------------------------------------------------+
 |                                                                            |
 +----------------------------------------------------------------------------*/

void PlayWave( unsigned int handle )
{

  if ( handle >= allewavs.size() ) return;
  SDL_PauseAudio(1);
  audio_pos = allewavs[ handle ].wav_buffer;
  audio_len = allewavs[ handle ].wav_length;
  SDL_PauseAudio(0);

}



/*-Typedefs-------------------------------------------------------------------+
 |                                                                            |
 +----------------------------------------------------------------------------*/

namespace GR
{
  namespace MODE
  {
    const DWORD MENU              = 0;
    const DWORD PLAY              = 1;
    const DWORD EDIT              = 2;
  };
  namespace CREATURE
  {
    namespace TYPE
    {
      const DWORD PLAYER          = 0;
      const DWORD COLLECT         = 1;
      const DWORD PLAYER_DEAD     = 2;
      const DWORD MAX_TYPES       = 3;
    };
    namespace FLAGS
    {
      const DWORD WALK_TO_TARGET  = 0x00000001;
    };
  };
  namespace GROUND
  {
    const DWORD EARTH           = 0;
    const DWORD WALL            = 1;
    const DWORD WATER           = 2;
    const DWORD DOOR_UP         = 3;
    const DWORD DOOR_DOWN       = 4;
    const DWORD BRIDGE_UP       = 5;
    const DWORD BRIDGE_DOWN     = 6;
    const DWORD MAX_TILES       = 7;
  };
  const DWORD WIDTH             = 20;
  const DWORD HEIGHT            = 15;
};

class CCreature
{
  public:

    float     m_fX,
              m_fY,
              m_fSpeed,
              m_fTargetX,
              m_fTargetY;

    DWORD     m_dwFlags,
              m_dwType;

    CCreature() :
      m_fX( 0.0f ),
      m_fY( 0.0f ),
      m_fSpeed( 50.0f ),
      m_fTargetX( 0.0f ),
      m_fTargetY( 0.0f ),
      m_dwFlags( 0 ),
      m_dwType( GR::CREATURE::TYPE::PLAYER )
    {
    }

};

typedef std::list<CCreature>  tListCreatures;

class CLevel
{
  public:

    DWORD     dwFeld[GR::WIDTH][GR::HEIGHT];
};



/*-Variablen------------------------------------------------------------------+
 |                                                                            |
 +----------------------------------------------------------------------------*/

int           iSoundGetReady,
              iSoundFupp,
              iSoundBling,
              iSoundYippie,
              iSoundSplash;

int	          iMouseX,
              iMouseY,
              iButtons;

float         fFlagX,
              fFlagY;

DWORD         dwLevel,
              dwMode,
              dwCollected = 0,
              dwEditFeld = GR::GROUND::WALL;

CLevel        theLevel;

bool          bShowFlag = false;

SDL_Surface		*surfTitle,
              *surfGround[GR::GROUND::MAX_TILES],
              *surfCreature[GR::CREATURE::TYPE::MAX_TYPES],
              *surfFlag;

tListCreatures  theCreatures;

CCreature       *pPlayer = NULL;


bool LoadLevel();



void Slock(SDL_Surface *screen)
{
  if ( SDL_MUSTLOCK(screen) )
  {
    if ( SDL_LockSurface(screen) < 0 )
    {
      return;
    }
  }
}

void Sulock(SDL_Surface *screen)
{
  if ( SDL_MUSTLOCK(screen) )
  {
    SDL_UnlockSurface(screen);
  }
}

void DrawPixel(SDL_Surface *screen, int x, int y,
                                    Uint8 R, Uint8 G, Uint8 B)
{
  if ( ( x < 0 )
  ||   ( y < 0 )
  ||   ( x >= 640 )
  ||   ( y >= 480 ) )
  {
    return;
  }
  Uint32 color = SDL_MapRGB(screen->format, R, G, B);
  switch (screen->format->BytesPerPixel)
  {
    case 1: // Assuming 8-bpp
      {
        Uint8 *bufp;
        bufp = (Uint8 *)screen->pixels + y*screen->pitch + x;
        *bufp = color;
      }
      break;
    case 2: // Probably 15-bpp or 16-bpp
      {
        Uint16 *bufp;
        bufp = (Uint16 *)screen->pixels + y*screen->pitch/2 + x;
        *bufp = color;
      }
      break;
    case 3: // Slow 24-bpp mode, usually not used
      {
        Uint8 *bufp;
        bufp = (Uint8 *)screen->pixels + y*screen->pitch + x * 3;
        if(SDL_BYTEORDER == SDL_LIL_ENDIAN)
        {
          bufp[0] = color;
          bufp[1] = color >> 8;
          bufp[2] = color >> 16;
        } else {
          bufp[2] = color;
          bufp[1] = color >> 8;
          bufp[0] = color >> 16;
        }
      }
      break;
    case 4: // Probably 32-bpp
      {
        Uint32 *bufp;
        bufp = (Uint32 *)screen->pixels + y*screen->pitch/4 + x;
        *bufp = color;
      }
      break;
  }
}



/*-SetGround------------------------------------------------------------------+
 |                                                                            |
 +----------------------------------------------------------------------------*/

void SetGround( int iX, int iY, DWORD dwType )
{

  if ( ( iX < 0 )
  ||   ( iY < 0 )
  ||   ( iX >= GR::WIDTH )
  ||   ( iY >= GR::HEIGHT ) )
  {
    return;
  }
  theLevel.dwFeld[iX][iY] = dwType;

}



/*-GetGround------------------------------------------------------------------+
 |                                                                            |
 +----------------------------------------------------------------------------*/

DWORD GetGround( int iX, int iY )
{

  if ( ( iX < 0 )
  ||   ( iY < 0 )
  ||   ( iX >= GR::WIDTH )
  ||   ( iY >= GR::HEIGHT ) )
  {
    return GR::GROUND::MAX_TILES;
  }
  return theLevel.dwFeld[iX][iY];

}



/*-InitGround-----------------------------------------------------------------+
 |                                                                            |
 +----------------------------------------------------------------------------*/

void InitGround()
{

  for ( int i = 0; i < GR::WIDTH; i++ )
  {
    for ( int j = 0; j < GR::HEIGHT; j++ )
    {
      theLevel.dwFeld[i][j] = GR::GROUND::EARTH;
    }
  }

  dwCollected = 0;

}



bool Init()
{

  srand( time( NULL ) );

  surfTitle = SDL_LoadBMP( "gfx\\title.bmp" );
  surfGround[GR::GROUND::EARTH]         = SDL_LoadBMP( "gfx\\earth.bmp" );
  surfGround[GR::GROUND::WALL]          = SDL_LoadBMP( "gfx\\wall.bmp" );
  surfGround[GR::GROUND::WATER]         = SDL_LoadBMP( "gfx\\water.bmp" );
  surfGround[GR::GROUND::DOOR_UP]       = SDL_LoadBMP( "gfx\\door_up.bmp" );
  surfGround[GR::GROUND::DOOR_DOWN]     = SDL_LoadBMP( "gfx\\door_down.bmp" );
  surfGround[GR::GROUND::BRIDGE_UP]     = SDL_LoadBMP( "gfx\\bridge.bmp" );
  surfGround[GR::GROUND::BRIDGE_DOWN]   = SDL_LoadBMP( "gfx\\bridgewater.bmp" );

  surfCreature[GR::CREATURE::TYPE::PLAYER]  = SDL_LoadBMP( "gfx\\creature.bmp" );
  surfCreature[GR::CREATURE::TYPE::COLLECT] = SDL_LoadBMP( "gfx\\collect.bmp" );
  surfCreature[GR::CREATURE::TYPE::PLAYER_DEAD] = SDL_LoadBMP( "gfx\\creature2.bmp" );
  
  surfFlag                              = SDL_LoadBMP( "gfx\\magnet.bmp" );

  for ( int i = 0; i < GR::GROUND::MAX_TILES; i++ )
  {
    surfGround[i] = SDL_DisplayFormat( surfGround[i] );
  }
  for ( i = 0; i < GR::CREATURE::TYPE::MAX_TYPES; i++ )
  {
    SDL_SetColorKey( surfCreature[i], SDL_SRCCOLORKEY | SDL_RLEACCEL, 0xff00ff );
    surfCreature[i] = SDL_DisplayFormat( surfCreature[i] );
  }

  SDL_SetColorKey( surfFlag, SDL_SRCCOLORKEY | SDL_RLEACCEL, 0xff00ff );
  surfFlag = SDL_DisplayFormat( surfFlag );

  iSoundYippie    = LoadWave( "sfx\\yippie.wav" );
  iSoundBling     = LoadWave( "sfx\\collect.wav" );
  iSoundGetReady  = LoadWave( "sfx\\getready.wav" );
  iSoundSplash    = LoadWave( "sfx\\splash.wav" );
  iSoundFupp      = LoadWave( "sfx\\fupp.wav" );

  SDL_AudioSpec obtained; //- dummy
  
  /* Open the audio device */
  if ( SDL_OpenAudio( &allewavs[iSoundYippie].wav_spec, &obtained ) < 0 )
  {
    return false;
  }

  InitGround();

  return true;

}



void SaveLevel()
{

  char    szName[MAX_PATH];
  wsprintf( szName, "level\\level%d.dat", dwLevel );

  FILE*   file;


  file = fopen( szName, "wb" );

  fwrite( &theLevel.dwFeld, sizeof( theLevel.dwFeld ), 1, file );
  int iDummy = theCreatures.size();
  fwrite( &iDummy, sizeof( iDummy ), 1, file );

  tListCreatures::iterator    it( theCreatures.begin() );
  while ( it != theCreatures.end() )
  {
    CCreature&   aCreature = *it;

    fwrite( &aCreature.m_dwType, sizeof( aCreature.m_dwType ), 1, file );
    fwrite( &aCreature.m_fX, sizeof( aCreature.m_fX ), 1, file );
    fwrite( &aCreature.m_fY, sizeof( aCreature.m_fY ), 1, file );


    it++;
  }
  fclose( file );

}



bool LoadLevel()
{
  
  bool    bResult = true;

  bShowFlag = false;
  theCreatures.clear();
  dwCollected = 0;

  char    szName[MAX_PATH];
  wsprintf( szName, "level\\level%d.dat", dwLevel );

  FILE*   file;

  file = fopen( szName, "rb" );
  if ( file != NULL )
  {
    fread( &theLevel.dwFeld, sizeof( theLevel.dwFeld ), 1, file );

    int iCount;
    fread( &iCount, sizeof( iCount ), 1, file );

    for ( int i = 0; i < iCount; i++ )
    {
      CCreature   aCreature;

      fread( &aCreature.m_dwType, sizeof( aCreature.m_dwType ), 1, file );
      fread( &aCreature.m_fX, sizeof( aCreature.m_fX ), 1, file );
      fread( &aCreature.m_fY, sizeof( aCreature.m_fY ), 1, file );

      if ( aCreature.m_dwType == GR::CREATURE::TYPE::COLLECT )
      {
        dwCollected++;
      }

      theCreatures.push_back( aCreature );
    }
    fclose( file );
  }
  else
  {
    bResult = false;
  }

  if ( theCreatures.size() == 0 )
  {
    CCreature   aCreature;
    aCreature.m_fX = 16;
    aCreature.m_fY = 16;
    theCreatures.push_back( aCreature );
  }

  pPlayer = &theCreatures.front();

  return bResult;

}



void ToggleDoors()
{

  for ( int i = 0; i < GR::WIDTH; i++ )
  {
    for ( int j = 0; j < GR::HEIGHT; j++ )
    {
      if ( theLevel.dwFeld[i][j] == GR::GROUND::DOOR_DOWN )
      {
        theLevel.dwFeld[i][j] = GR::GROUND::DOOR_UP;
      }
      else if ( theLevel.dwFeld[i][j] == GR::GROUND::DOOR_UP )
      {
        theLevel.dwFeld[i][j] = GR::GROUND::DOOR_DOWN;
      }
    }
  }

}



void ToggleBridges()
{

  for ( int i = 0; i < GR::WIDTH; i++ )
  {
    for ( int j = 0; j < GR::HEIGHT; j++ )
    {
      if ( theLevel.dwFeld[i][j] == GR::GROUND::BRIDGE_DOWN )
      {
        theLevel.dwFeld[i][j] = GR::GROUND::BRIDGE_UP;
      }
      else if ( theLevel.dwFeld[i][j] == GR::GROUND::BRIDGE_UP )
      {
        theLevel.dwFeld[i][j] = GR::GROUND::BRIDGE_DOWN;
      }
    }
  }

}



void DrawSurface( SDL_Surface *target, int iX, int iY, SDL_Surface *image )
{

	SDL_Rect dest;
	dest.x = iX;
	dest.y = iY;
	SDL_Rect dest2;
	dest2.x = 0;
	dest2.y = 0;
	dest2.w = image->w;
	dest2.h = image->h;
	SDL_BlitSurface( image, &dest2, target, &dest );

}



void DisplayFrame( SDL_Surface *screen )
{

  Slock( screen );

  if ( dwMode == GR::MODE::MENU )
  {
    DrawSurface( screen, 0, 0, surfTitle );
  }
  else if ( ( dwMode == GR::MODE::PLAY )
  ||        ( dwMode == GR::MODE::EDIT ) )
  {
    for ( int i = 0; i < GR::WIDTH; i++ )
    {
      for ( int j = 0; j < GR::HEIGHT; j++ )
      {
        DrawSurface( screen, i * 32, j * 32, surfGround[theLevel.dwFeld[i][j]] );
      }
    }

    {
      tListCreatures::iterator    it( theCreatures.begin() );

      while ( it != theCreatures.end() )
      {
        CCreature&  aCreature = *it;

        DrawSurface( screen, aCreature.m_fX - 8, aCreature.m_fY - 8, surfCreature[aCreature.m_dwType] );

        /*
        if ( aCreature.m_dwFlags & GR::CREATURE::FLAGS::WALK_TO_TARGET )
        {
          DrawPixel( screen, aCreature.m_fTargetX, aCreature.m_fTargetY, 255, 255, 255 );
        }
        */
 
        it++;
      }
    }

    if ( bShowFlag )
    {
      DrawSurface( screen, fFlagX - 8, fFlagY - 16, surfFlag );
    }

    if ( dwMode == GR::MODE::EDIT )
    {
      DrawSurface( screen, iMouseX, iMouseY, surfGround[dwEditFeld] );
    }
  }

  Sulock( screen );
  SDL_Flip( screen );

}



void CheckCreatureCollission()
{

  tListCreatures::iterator    it( theCreatures.begin() );

  while ( it != theCreatures.end() )
  {
    CCreature&  aCreature = *it;

    if ( aCreature.m_dwType == GR::CREATURE::TYPE::PLAYER )
    {
      // test player against all the other things

      tListCreatures::iterator    it2( theCreatures.begin() );
      while ( it2 != theCreatures.end() )
      {
        CCreature&  bCreature = *it2;

        if ( bCreature.m_dwType == GR::CREATURE::TYPE::COLLECT )
        {
          if ( ( abs( bCreature.m_fX - aCreature.m_fX ) < 10.0f )
          &&   ( abs( bCreature.m_fY - aCreature.m_fY ) < 10.0f ) )
          {
            // collect it!
            it2 = theCreatures.erase( it2 );

            if ( --dwCollected )
            {
              PlayWave( iSoundBling );
            }
            else
            {
              PlayWave( iSoundYippie );
            }
          }
          else
          {
            it2++;
          }
        }
        else
        {
          it2++;
        }
      }

    }

    it++;
  }

}



void UpdateCreatures( float fTime )
{

  tListCreatures::iterator    it( theCreatures.begin() );

  while ( it != theCreatures.end() )
  {
    CCreature&  aCreature = *it;

    DWORD   dwGround = GetGround( aCreature.m_fX / 32.0f, aCreature.m_fY / 32.0f );

    if ( ( bShowFlag )
    &&   ( aCreature.m_dwType != GR::CREATURE::TYPE::PLAYER )
    &&   ( aCreature.m_dwType != GR::CREATURE::TYPE::PLAYER_DEAD ) )
    {
      aCreature.m_dwFlags |= GR::CREATURE::FLAGS::WALK_TO_TARGET;
      aCreature.m_fTargetX = fFlagX;
      aCreature.m_fTargetY = fFlagY;
    }
    if ( aCreature.m_dwFlags & GR::CREATURE::FLAGS::WALK_TO_TARGET )
    {
      float   fDeltaX = aCreature.m_fTargetX - aCreature.m_fX,
              fDeltaY = aCreature.m_fTargetY - aCreature.m_fY,
              fAngle,
              fGroundFactor = 1.0f;

      fAngle = atan2( fDeltaY, fDeltaX );

      int   iTargetX = aCreature.m_fX + fTime * fGroundFactor * aCreature.m_fSpeed * cos( fAngle ),
            iTargetY = aCreature.m_fY + fTime * fGroundFactor * aCreature.m_fSpeed * sin( fAngle );

      if ( ( GetGround( iTargetX / 32, iTargetY / 32 ) == GR::GROUND::WATER )
      ||   ( GetGround( iTargetX / 32, iTargetY / 32 ) == GR::GROUND::BRIDGE_DOWN ) )
      {
        PlayWave( iSoundSplash );
        it = theCreatures.erase( it );

        pPlayer->m_dwType = GR::CREATURE::TYPE::PLAYER_DEAD;
        continue;
      }
      if ( ( GetGround( iTargetX / 32, iTargetY / 32 ) != GR::GROUND::WALL )
      &&   ( GetGround( iTargetX / 32, iTargetY / 32 ) != GR::GROUND::DOOR_UP ) )
      {
        aCreature.m_fX += fTime * fGroundFactor * aCreature.m_fSpeed * cos( fAngle );
        aCreature.m_fY += fTime * fGroundFactor * aCreature.m_fSpeed * sin( fAngle );
      }
    }

    it++;
  }

}



int main(int argc, char *argv[])
{

	Uint8 *keys;

  if ( SDL_Init(SDL_INIT_AUDIO|SDL_INIT_VIDEO) < 0 )
  {
    printf("Unable to init SDL: %s\n", SDL_GetError());
    exit(1);
  }
  atexit(SDL_Quit);

  SDL_WM_SetCaption( "Magnetic Fields", "" );

  SDL_Surface *screen;
  screen=SDL_SetVideoMode(640,480,16,SDL_HWSURFACE|SDL_DOUBLEBUF);
  if ( screen == NULL )
  {
    printf("Unable to set 640x480 video: %s\n", SDL_GetError());
    exit(1);
  }

  if ( !Init() )
  {
	  exit( 1 );
  }



  int done=0;

  while(done == 0)
  {
    SDL_Event event;

    while ( SDL_PollEvent(&event) )
    {
      if ( event.type == SDL_QUIT )  {  done = 1;  }

      if ( event.type == SDL_KEYDOWN )
      {
        if ( event.key.keysym.sym == SDLK_ESCAPE ) { done = 1; }
        if ( event.key.keysym.sym == SDLK_e ) 
        { 
          dwLevel = 1;
          InitGround();
          LoadLevel();
          dwMode = GR::MODE::EDIT;
        }
        if ( dwMode == GR::MODE::PLAY )
        {
          if ( event.key.keysym.sym == SDLK_KP_PLUS ) 
          {
            dwLevel++;
            InitGround();
            LoadLevel();
          }
          if ( event.key.keysym.sym == SDLK_KP_MINUS ) 
          {
            if ( dwLevel > 1 )
            {
              dwLevel--;
              InitGround();
              LoadLevel();
            }
          }
        }
        if ( dwMode == GR::MODE::EDIT )
        {
          if ( event.key.keysym.sym == SDLK_KP_PLUS ) 
          {
            dwLevel++;
            InitGround();
            LoadLevel();
          }
          if ( event.key.keysym.sym == SDLK_KP_MINUS ) 
          {
            if ( dwLevel > 1 )
            {
              dwLevel--;
              InitGround();
              LoadLevel();
            }
          }
          if ( event.key.keysym.sym == SDLK_s ) 
          {
            pPlayer->m_fX = iMouseX;
            pPlayer->m_fY = iMouseY;
          }
          if ( event.key.keysym.sym == SDLK_b ) 
          {
            CCreature   newCreature;

            newCreature.m_dwType = GR::CREATURE::TYPE::COLLECT;
            newCreature.m_fX = iMouseX;
            newCreature.m_fY = iMouseY;

            theCreatures.push_back( newCreature );
          }
          if ( event.key.keysym.sym == SDLK_F2 ) 
          {
            SaveLevel();
          }
          if ( event.key.keysym.sym == SDLK_F4 ) 
          {
            dwMode = GR::MODE::MENU;
          }
        }
        if ( dwMode == GR::MODE::MENU )
        {
          if ( event.key.keysym.sym == SDLK_p )
          {
            dwLevel = 1;
            if ( LoadLevel() )
            {
              dwMode = GR::MODE::PLAY;
              PlayWave( iSoundGetReady );
            }
          }
        }
      }
    }

	iButtons = SDL_GetMouseState( &iMouseX, &iMouseY );

	{
		static int	iLastTicks = GetTickCount();

		float	fZeitFaktor = ( (float)( GetTickCount() - iLastTicks ) ) / 1000.0f;

		iLastTicks = GetTickCount();

    if ( dwMode == GR::MODE::EDIT )
    {
  		if ( SDL_BUTTON( 1 ) & iButtons )
	  	{
        SetGround( iMouseX / 32, iMouseY / 32, dwEditFeld );
      }
    }

    {
      static bool bButtonReleased = true,
                  bButton2Released = true;
      if ( SDL_BUTTON( 1 ) & iButtons )
      {
        if ( dwMode == GR::MODE::PLAY )
        {
          if ( pPlayer->m_dwType == GR::CREATURE::TYPE::PLAYER_DEAD )
          {
            LoadLevel();
          }
          else if ( bButtonReleased )
          {
            bShowFlag = true;
            fFlagX = iMouseX;
            fFlagY = iMouseY;
            PlayWave( iSoundFupp );
          }
        }
        bButtonReleased = false;
      }
      else
      {
        bButtonReleased = true;
      }
      if ( SDL_BUTTON( 3 ) & iButtons )
      {
        if ( dwMode == GR::MODE::PLAY )
        {
          if ( bButton2Released )
          {
            ToggleDoors();
          }
        }
        if ( dwMode == GR::MODE::EDIT )
        {
          if ( bButton2Released )
          {
            dwEditFeld = ( dwEditFeld + 1 ) % GR::GROUND::MAX_TILES;
          }
        }
        bButton2Released = false;
      }
      else
      {
        bButton2Released = true;
      }

      // bridges
      {
        static int	iLastBridgesTicks = GetTickCount();

        if ( iLastBridgesTicks / 1000 != GetTickCount() / 1000 )
        {
          ToggleBridges();
        }
        iLastBridgesTicks = GetTickCount();
      }
    }

		keys = SDL_GetKeyState( NULL );

    if ( keys[SDLK_SPACE] )
    {
      fZeitFaktor *= 10.0f;
    }

		DisplayFrame( screen );

    CheckCreatureCollission();
    UpdateCreatures( fZeitFaktor );

    if ( dwMode == GR::MODE::PLAY )
    {
      if ( dwCollected == 0 )
      {
        // collected all coins
        dwLevel++;
        if ( !LoadLevel() )
        {
          dwMode = GR::MODE::MENU;
        }
      }
    }
	}

  }

  return 0;
}

